/*
Hex by Ben G
Copyright (c) 2006 Ben Gilmore

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

var
	const
		n = 11 // number of hex in the middle row
		WIDTH = 30 // width, in pixels, of a hex
		HEIGHT = 31 // height, in pixels, of a hex

world
	hub = "BenG.Hex"
	view = "12x18"
	New()
		..()
		var
			Y_offset //beginning x offset
			X_offset = -9 // beginning y offset
			totalX = 5 // hex building starts at this x coordinate
			totalY = 2 // hex building starts at this y coordinate

		generateWinLists() // this was easier to write out than all the possibilities, also, it allows for the creation of a larger board seamlessly

		for(var/row=1 to n*2-1) //generates a list of all the possible hex coordinates (these are assigned to actual hex tiles later)
			for(var/col=1 to (row<=n)?(row):(n-(row-n)))
				hexes += "[row],[col]"
				hexes["[row],[col]"] = list()

		for(var/row=1 to n*2-1) //the actual procedure for rendering the board - it's pretty complex, so I won't go in to detail
			for(var/i=1 to (row<=n)?(row):(n-(row-n)))
				var
					new_pixelX = ((row%2)?(i*WIDTH):(i*WIDTH+WIDTH/2)) - X_offset
					currentX = totalX
					currentY = totalY
					obj/hex/H = new(row, i)

				if(new_pixelX >= 32)
					currentX += divideEvenly(new_pixelX, 32)
					new_pixelX -= divideEvenly(new_pixelX, 32) * 32

				H.loc = locate(currentX, currentY, 1)
				H.pixel_x = new_pixelX
				H.pixel_y = Y_offset

			Y_offset += round(HEIGHT*3/4)+1
			if(Y_offset >= 32)
				totalY += divideEvenly(Y_offset, 32)
				Y_offset -= 32*divideEvenly(Y_offset, 32)
			if(row<n)
				X_offset += ((row%2)?(WIDTH):(0))
			else
				X_offset -= ((row%2)?(0):(WIDTH))


proc
	hexAdjacent(var/obj/hex/A, var/obj/hex/B) //procedure to determine if two hex tiles are adjacent
		if((A.row == B.row)&&((B.column == A.column+1)||(B.column == A.column-1)))
			return TRUE
		if((A.column == B.column)&&(abs(A.row-B.row)==1))
			return TRUE
		if((A.row < n)||((A.row == n)&&(B.row < n)))
			if((A.row+1 == B.row)&&(A.column+1 == B.column))
				return TRUE
			if((A.row-1 == B.row)&&(A.column-1 == B.column))
				return TRUE
		if((A.row > n)||((A.row == n)&&(B.row > n)))
			if((A.row-1 == B.row)&&(A.column+1 == B.column))
				return TRUE
			if((A.row+1 == B.row)&&(A.column-1 == B.column))
				return TRUE
		return FALSE

	determinePaths(var/obj/hex/A)
		var/list
			aPath = list("[A.row],[A.column]")
			colorPaths = paths["[A.icon_state]"]
		for(var/row=A.row-1 to A.row+1)//there are several mathematical
				//computations that go in to determining if two hex are adjacent - I didn't want to bog down this procedure with
				// them, so I simply go through a list of every tile that *could* be adjacent and use hexAdjacent() to determine
				// if they actually are
			for(var/col=A.column-1 to A.column+1)
				if((row==A.row)&(col==A.column)) continue //same tile
				if((row<=0)||(col<=0)||(col>((row<=n)?(row):(n-(row-n))))||(row>(n*2-1))) continue //not a tile on the board
				var/obj/hex/temp = hexes["[row],[col]"]
				/*
				The below method I use for finding paths is somewhat crative.  Every tile, when placed in an isolated
				position, is its own path.  When paths merge (that is, two or more paths become adjacent to one another), their
				lists are merged, creating one path.  I then check the path for at least one tile from each target side of the
				board.  If the path has both, the player wins.
				*/
				if((!("[row],[col]" in aPath))&&(A.icon_state==temp.icon_state)&&(hexAdjacent(A, temp)))
					for(var/l in colorPaths)
						var/list/path = l
						for(var/a in path)
							if(a == "[row],[col]")
								if(istype(path, /list))
									aPath += path
								else
									aPath += list(path)
								colorPaths -= list(l)
								break
		colorPaths += list(aPath) //add the new path to the global list
		var/list/colorWins = wins["[A.icon_state]"] //used to determine if the path stretches the board
		for(var/i in colorWins[1])
			if(i in aPath)
				for(var/j in colorWins[2])
					if(j in aPath)
						Win(A.icon_state)
						break

	divideEvenly(n, n2) //simple enough
		return round(n/n2)