/*
   _____      _             __
  |_   _|    | |           / _|
    | | _ __ | |_ ___ _ __| |_ __ _  ___ ___
    | || '_ \| __/ _ \ '__|  _/ _` |/ __/ _ \
   _| || | | | ||  __/ |  | || (_| | (_|  __/
   \___/_| |_|\__\___|_|  |_| \__,_|\___\___|


  ___  ____       _
  |  \/  (_)     (_)
  | .  . |_ _ __  _ _ __ ___   __ _ _ __
  | |\/| | | '_ \| | '_ ` _ \ / _` | '_ \
  | |  | | | | | | | | | | | | (_| | |_) |
  \_|  |_/_|_| |_|_|_| |_| |_|\__,_| .__/
                                   | |
                                   |_|

    LIBRARY   8.26.10   BY DARKCAMPAINGER

        VERSION 2.0 - RELEASE 7.30.11


    See included demo for instructions

*/

#define DC_MINIMAP_DATA_LABEL "dc_Minimap_SDF72L21DSP"

var
	list/dc_MinimapList = list() // "Minimap ID" = dc_Minimap object
	dc_MinimapPrefix = "dc_Minimap"


client
	var/list/dc_Minimaps //list of dc_Minimap objects with their index relating to their pane ID. null spaces where panes are open
	var/list/dc_MinimapData = list() //Stores existing controls for reboots

	proc
		dc_MinimapPrepare()

			//Check for leftover Minimap controls from reboot
			if(winexists(src,DC_MINIMAP_DATA_LABEL))
				var/list/controls=params2list(winget(src,DC_MINIMAP_DATA_LABEL,"text"))

				var/panes = text2num(controls[1])
				if(panes) {dc_Minimaps=list();dc_Minimaps.len=panes;}

				var/settings = ""
				for(var/i = 2 to controls.len)
					settings+="[controls[i]].parent='';"

				winset(src,null,settings)

			else //Otherwise set up corners
				var/windowsText = winget(src, null, "windows")
				var/windows = params2list(windowsText)
				if(!istype(windows,/list)) windows=list(windows)

				//Make sure no windows are maximized
				var/settings
				for(var/W in windows)
					settings+="[W].is-maximized=false;"
				winset(src, null, settings)

				//Get all of their sizes
				var/list/sizes = params2list(winget(src, windowsText, "size"))
				if(!istype(sizes,/list)) sizes=list(sizes)

				//Create the data-holding label
				var/dataWindow = min(windows)
				winset(src, DC_MINIMAP_DATA_LABEL, "parent=[dataWindow];type=label;is-visible=false;size=1x1;text=\"0\";")


				//Create all of the corner-labels
				if(sizes.len>1) //Mutiple windows
					for(var/W in windows)
						var
							S = sizes["[W].size"]
							D = findtext(S,"x")
						winset(src, "[dc_MinimapPrefix]_[W]_Corner", "parent=[W];type=label;is-visible=false;size=1x1;anchor1=100,100;pos=[copytext(S,1,D)],[copytext(S,D+1)];")
				else //Just one window
					var
						W = windows[1]
						S = sizes[1]
						D = findtext(S,"x")
					winset(src, "[dc_MinimapPrefix]_[W]_Corner", "parent=[W];type=label;is-visible=false;size=1x1;anchor1=100,100;pos=[copytext(S,1,D)],[copytext(S,D+1)];")

		dc_MinimapDataString() //Returns data to store in DC_MINIMAP_DATA_LABEL text field. List of controls that needs to be deleted
			return "[DC_MINIMAP_DATA_LABEL].text=\"[(dc_Minimaps) ? dc_Minimaps.len : 0];[list2params(dc_MinimapData)]\";"


proc
	dc_MinimapRegister(A,T,F) //Set F to the atom you want the Minimap to follow (instead of the client's mob)
		var {client/C;mob/M}
		if(ismob(A)) {M=A;C=M.client}
		else if(istype(A,/client)) {C=A;M=C.mob}

		if(C && T)
			var/dc_Minimap/D
			if(T in dc_MinimapList)
				D = dc_MinimapList[T]
				if(D) D.Register(C,F)
				else CRASH("dc_Minimap '[T]' reference in dc_MinimapList lost!")
			else CRASH("dc_Minimap '[T]' hasn't been created yet!")
		else CRASH("Improper object '[A]' or Minimap ID '[T]' passed to Register. Expected client (or mob with client) and Minimap ID.")

	dc_MinimapUnregister(A,T,keep=0) //Set keep to true to prevent the MiniMap interface from being removed
		var {client/C;mob/M}
		if(ismob(A)) {M=A;C=M.client}
		else if(istype(A,/client)) {C=A;M=C.mob}

		if(C)
			if(T)
				var/dc_Minimap/D
				if(T in dc_MinimapList)
					D = dc_MinimapList[T]
					if(D) D.Unregister(C,keep)
					else CRASH("dc_Minimap '[T]' reference in dc_MinimapList lost!")
				else CRASH("dc_Minimap '[T]' hasn't been created yet!")
			else
				for(var/dc_Minimap/D in C.dc_Minimaps)
					if(D) D.Unregister(C,keep)
		else CRASH("Improper object '[A]' passed to Unregister. Expected client (or mob with client).")

	dc_MinimapUpdate(A,T,X,Y)
		var {client/C;mob/M}
		if(ismob(A)) {M=A;C=M.client}
		else if(istype(A,/client)) {C=A;M=C.mob}

		if(C)
			if(T)
				var/dc_Minimap/D
				if(T in dc_MinimapList)
					D = dc_MinimapList[T]
					if(D) D.Update(M,X,Y)
					else CRASH("dc_Minimap '[T]' reference in dc_MinimapList lost!")
				else CRASH("dc_Minimap '[T]' hasn't been created yet!")
			else
				for(var/dc_Minimap/D in C.dc_Minimaps)
					if(D) D.Update(M,X,Y)
		else CRASH("Improper object '[A]' passed to MinimapUpdate. Expected client (or mob with client).")

	dc_MinimapRemove(A,T)
		var {client/C;mob/M}
		if(ismob(A)) {M=A;C=M.client}
		else if(istype(A,/client)) {C=A;M=C.mob}

		if(C)
			if(T)
				var/dc_Minimap/D
				if(T in dc_MinimapList)
					D = dc_MinimapList[T]
					if(D) D.Remove(C)
					else CRASH("dc_Minimap '[T]' reference in dc_MinimapList lost!")
				else CRASH("dc_Minimap '[T]' hasn't been created yet!")
			else
				for(var/dc_Minimap/D in C.dc_Minimaps)
					if(D) D.Remove(C)
		else CRASH("Improper object '[A]' passed to MinimapRemove. Expected client (or mob with client).")