dc_Minimap
	var
		id                           // Unique ID for this Minimap
		parent                       // The ID of the interface window or pane that will hold the Minimap

		x1;y1;x2;y2;z                // Coordinates on map that Minimap covers. (x1,y1,z) to (x2,y2,z)

		viewX;viewY                  // Position of Minimap on the interface
		                             //   Set to Number for absolute positioning: viewX=50
		                             //   Set to Text for relative positioning: viewX="EAST+50";viewY="SOUTH+20"
		                             //     Similar to screen_loc format, but in pixels only
		                             //     (useful for resizable windows)
		viewWidth                    // Size of the Minimap on the interface
		viewHeight
		viewAnchor = "none"          // In the form of "[X%],[Y%]". Sets the Minimap's Anchor for window resizes
		                             //   (ex: "0,0" for the top-left corner, "0,100" for the bottom-left corner, ect)

		imageWidth                   // Dimensions of Minimap image. Set this *OR* scaleX/Y. Takes precedence over scaleX/Y
		imageHeight                  //   (image pixel dimensions must be multiples of the turf area dimensions)
		image                        // Pregenerated image file with Minimap. Leave null for runtime generation

		scaleX                       // Size of square (pixels) on image each world turf takes. Set this *OR* imageWidth/Height
		scaleY                       //   (must be integers)

		backgroundColor = "#000000"  // Color of area BEHIND Minimap
		backgroundImage = null       // Image displayed behind Minimap: 'Fog.jpg'
		backgroundMode = "tile"      // Image mode: tile, stretch, or center
		border = "none"              // Control border property: none, line, or sunken
		padding = 0                  // Size of padded border around Minimap (pixels)
		paddingColor = "#000000"     // Color of the padded border
		paddingImage = null          // Image file for padded border: 'Frame.jpg'
		paddingMode = "center"       // Image mode: tile, stretch, or center
		paddingBorder = "none"       // Control border property: none, line, or sunken

		marker                       // Path or list of paths pointing to dc_MinimapMarker types

		delay = 10                   // How often registered clients are updated

		hideEdges = 0                // Prevents the Minimap from showing its edges

		// Determined at runtime (do not set!)

		imageGenerated               // Flag for if image was generated at runtime

		areaWidth                    // Area Minimap covers, in tiles
		areaHeight
		offsetX                      // Bottom-left corner of Minimap's area
		offsetY

		list/clients                 // Associative list of clients (client = [atom following])
		                             //   If no associated atom, defaults to client.mob
		updateLoopID = 0             // Ensures only one update loop is running at any time

	New(ID, Parent, X1,Y1,X2,Y2,Z, ViewX, ViewY, ViewWidth, ViewHeight, ViewAnchor, ImageWidth,ImageHeight,Image, ScaleX,ScaleY, BackgroundColor,BackgroundImage,BackgroundMode, Border, Padding, PaddingColor, PaddingImage, PaddingMode, PaddingBorder, Marker, Delay, HideEdges)
		if(ID) id=ID
		if(Parent) parent=Parent
		if(X1) x1=X1
		if(Y1) y1=Y1
		if(X2) x2=X2
		if(Y2) y2=Y2
		if(Z) z=Z
		if(ViewX) viewX=ViewX
		if(ViewY) viewY=ViewY
		if(ViewWidth) viewWidth=ViewWidth
		if(ViewHeight) viewHeight=ViewHeight
		if(ViewAnchor) viewAnchor=ViewAnchor
		if(ImageWidth) imageWidth=ImageWidth
		if(ImageHeight) imageHeight=ImageHeight
		if(Image) image=Image
		if(ScaleX) scaleX = ScaleX
		if(ScaleY) scaleY = ScaleY
		if(BackgroundColor) backgroundColor=BackgroundColor
		if(BackgroundImage) backgroundImage=BackgroundImage
		if(BackgroundMode) backgroundMode=BackgroundMode
		if(Border) border=Border
		if(Padding) padding=Padding
		if(PaddingColor) paddingColor=PaddingColor
		if(PaddingImage) paddingImage=PaddingImage
		if(PaddingMode) paddingMode=PaddingMode
		if(PaddingBorder) paddingBorder=PaddingBorder
		if(Marker) marker=Marker
		if(Delay) delay=Delay
		if(HideEdges) hideEdges = HideEdges


		if(!id) CRASH("Minimap does not have an ID set")
		if(!x1 || !y1 || !x2 || !y2 || !z || x1>x2 || y1>y2 || x1<1 || x2<1 || y1<1 || y2<1 || z<1) CRASH("Bad Minimap coordinates: ([x1],[y1],[z]) to ([x2],[y2],[z])")
		if((!imageWidth && !scaleX) || (!imageHeight && !scaleY)) CRASH("Minimap image dimensions not set! Must set imageWidth/Height or scaleX/Y!")
		if(!viewWidth || !viewHeight || viewX==null || viewY==null) CRASH("Minimap location and size not set!")

		areaWidth = x2-x1+1
		areaHeight = y2-y1+1
		offsetX = x1-1
		offsetY = y1-1

		//Determine imageWidth/Height or scaleX/Y, whichever is missing
		if(!imageWidth) imageWidth = scaleX*areaWidth
		else scaleX = imageWidth/areaWidth
		if(!imageHeight) imageHeight = scaleY*areaHeight
		else scaleY = imageHeight/areaHeight

		//If image hasn't been passed, generate it
		if(!image)
			if(round(scaleX)!=scaleX || round(scaleY)!=scaleY) CRASH("Minimap generator does not currently support fractional scaleX/Y (image pixel dimensions must both be multiples of the turf area dimensions, and cannot be smaller)")
			image = fcopy_rsc(dc_GenerateMinimap(x1,y1,x2,y2,z,scaleX,scaleY))
			if(image) imageGenerated = 1
			else CRASH("Failed to generate Minimap image! dc_GenerateMinimap([x1],[y1],[x2],[y2],[z],[scaleX],[scaleY])")

		//Create all markers
		if(ispath(marker)) marker = new marker()
		else if(istype(marker,/list))
			var/list/L = list()
			for(var/p in marker)
				if(ispath(p)) L+= new p()
				else if(istype(p,/dc_MinimapMarker)) L+=p
			marker=L

		//Register with global Minimap list
		if(id in dc_MinimapList) CRASH("Minimap id '[id]' has already been registered!")
		dc_MinimapList[id] = src

		return 1 //Creation successful!

	Del()
		updateLoopID++
		for(var/client/C)
			if(C.dc_Minimaps && (src in C.dc_Minimaps))
				Remove(C)

		dc_MinimapList[id] = null
		dc_MinimapList -= id

		..()

	proc
		Register(A,atom/F)
			var {client/C;mob/M}
			if(ismob(A)) {M=A;C=M.client}
			else if(istype(A,/client)) {C=A;M=C.mob}

			if(C)
				var/startLoop

				if(!clients)
					clients=list()
					startLoop=1

				if(!istype(F))
					F=null

				clients[C] = F

				if(startLoop)
					UpdateLoop()
				else
					if(!F) F=C.mob
					Update(C,F.x,F.y)

			else CRASH("Improper object '[A]' passed to Register. Expected client, or mob with client.")

		Unregister(A,keep=0)
			var {client/C;mob/M}
			if(ismob(A)) {M=A;C=M.client}
			else if(istype(A,/client)) {C=A;M=C.mob}

			if(C)
				clients[C] = null
				clients-=C
				if(!clients.len) clients=null

				if(!keep)
					Remove(C)

			else CRASH("Improper object '[A]' passed to Unregister. Expected client, or mob with client.")

		UpdateLoop()
			spawn()
				var/loopID = ++updateLoopID
				var/atom/F
				while(updateLoopID==loopID && clients && clients.len)
					for(var/client/C in clients)
						F = clients[C] //If a specific atom was specified, use that. Otherwise, use the client's mob
						if(!F) F=C.mob
						Update(C,F.x,F.y)
					sleep(delay)

		Initialize(A)
			var {client/C;mob/M}
			if(ismob(A)) {M=A;C=M.client}
			else if(istype(A,/client)) {C=A;M=C.mob}

			if(C)

				//Register the Minimap with the client
				if(!C.dc_Minimaps) C.dc_Minimaps=list()
				else if(src in C.dc_Minimaps) {world.log<<"Tried to double-initialize Minimap '[id]' on [C]";return}

				//Look for old panes to reuse
				var/create=1
				for(var/i = 1 to C.dc_Minimaps.len)
					if(C.dc_Minimaps[i] == null) //Look for unused panes
						C.dc_Minimaps[i] = src
						create=0
						break

				if(create) //Need to create new pane
					C.dc_Minimaps[++C.dc_Minimaps.len] = src

				//Send any images
				if(!imageGenerated) C<<browse_rsc(image)
				if(backgroundImage) C<<browse_rsc(backgroundImage)
				if(padding && paddingImage) C<<browse_rsc(paddingImage)

				//Set up relative positioning
				var/vX=viewX,vY=viewY,vR,vN,size
				if(istext(vX))
					vN = findtext(vX,"+")
					if(!vN) vN = findtext(vX,"-")
					if(vN) vN=text2num(copytext(vX,vN))
					vR=text2ascii(vX,1)
					if(vR==69) vR=1        //69=E -> EAST
					else if(vR==67) vR=0.5 //67=C -> CENTER
					else vR=0              //87=W -> WEST

					if(vR)
						if(!size) size = winget(C, "[dc_MinimapPrefix]_[parent]_Corner", "pos")
						vX = vN + vR*text2num(copytext(size,1,findtext(size,",")))
					else vX=vN
				if(istext(vY))
					vN = findtext(vY,"+")
					if(!vN) vN = findtext(vY,"-")
					if(vN) vN=text2num(copytext(vY,vN))
					vR=text2ascii(vY,1)
					if(vR==83) vR=1        //83=S -> SOUTH
					else if(vR==67) vR=0.5 //67=C -> CENTER
					else vR=0              //78=N -> NORTH

					if(vR)
						if(!size) size = winget(C, "[dc_MinimapPrefix]_[parent]_Corner", "pos")
						vY = vN + vR*text2num(copytext(size,findtext(size,",")+1))
					else vY=vN

				//Create and set up the interface pane/controls
				var/w = "[dc_MinimapPrefix][C.dc_Minimaps.Find(src)]"
				if(padding)
					winset(C, "[w]_padding", "type=label;parent=[parent];pos=[vX-padding],[vY-padding];size=[viewWidth+2*padding]x[viewHeight+2*padding];anchor1=[viewAnchor];border=[paddingBorder];background-color=[paddingColor];[paddingImage ? "image='[paddingImage]';" : ""][paddingMode ? "image-mode=[paddingMode];" : ""]")

					C.dc_MinimapData+="[w]_padding"
				C.dc_MinimapData+="[w]_child"
				C.dc_MinimapData+="[w]_label"
				winset(C, null, C.dc_MinimapDataString())

				if(create) winclone(C, "pane", "[w]_pane")
				winset(C, "[w]_pane","size=[viewWidth],[viewHeight];background-color=[backgroundColor];image=[backgroundImage];image-mode=[backgroundMode]")
				winset(C, "[w]_child", "parent=[parent];type=child;pos=[vX],[vY];size=[viewWidth]x[viewHeight];anchor1=[viewAnchor];border=[border];left=[w]_pane;")
				winset(C, "[w]_label", "parent=[w]_pane;type=label;size=[imageWidth]x[imageHeight];anchor1=0,0;anchor2=100,100;image='[imageGenerated ? "\ref[image]" : image]';")

				//Create any Marker Labels
				if(istype(marker,/list))
					for(var/dc_MinimapMarker/D in marker)
						D.Initialize(C,src)
				else if(istype(marker,/dc_MinimapMarker))
					var/dc_MinimapMarker/D = marker
					D.Initialize(C,src)

			else if(istype(A,/list))
				for(var/B in A)
					Initialize(B)

			else CRASH("Improper object '[A]' passed to Initialize. Expected client, or mob with client.")

		Update(A,X,Y)
			var {client/C;mob/M}
			if(ismob(A)) {M=A;C=M.client}
			else if(istype(A,/client)) {C=A;M=C.mob}

			if(M)
				if(X==null) X=M.x
				if(Y==null) Y=M.y

			if(C && X!=null && Y!=null)
				X = min(max(X,x1),x2)
				Y = min(max(Y,y1),y2)

				if(!C.dc_Minimaps || !(src in C.dc_Minimaps))
					Initialize(C)

				if(C.dc_Minimaps && (src in C.dc_Minimaps))
					var
						posX = round(viewWidth/2-(X-offsetX-1)*scaleX-1,1)
						posY = round(viewWidth/2-(imageHeight-(Y-offsetY-1)*scaleY-1),1)
					if(hideEdges)
						posX = min(max(posX,-imageWidth+viewWidth),0)
						posY = min(max(posY,-imageHeight+viewHeight),0)
					winset(C,"[dc_MinimapPrefix][C.dc_Minimaps.Find(src)]_label","pos=[posX],[posY];")
				else CRASH("Update called on Minimap before being Initialized for client '[C]'. Auto-Initialize failed.")
			else CRASH("Improper object '[A]' passed to Update. Expected mob, or client with mob.")

		Remove(A)
			var {client/C;mob/M}
			if(ismob(A)) {M=A;C=M.client}
			else if(istype(A,/client)) {C=A;M=C.mob}

			if(C)
				//Unregister is registered for auto updates
				if(clients && (C in clients))
					Unregister(C,1) //Set keep to 1 so Remove isn't called twice

				//Remove the Minimap from the client
				if(C.dc_Minimaps && (src in C.dc_Minimaps))

					//Delete all marker labels
					if(istype(marker,/dc_MinimapMarker))
						var/dc_MinimapMarker/D = marker
						D.Remove(C,src)
					else if(istype(marker,/list))
						for(var/dc_MinimapMarker/D in marker)
							D.Remove(C,src)

					//Delete padding, child, and label controls
					var/w = "[dc_MinimapPrefix][C.dc_Minimaps.Find(src)]"
					C.dc_MinimapData-="[w]_label";C.dc_MinimapData-="[w]_child"
					if(padding) C.dc_MinimapData-="[w]_padding"
					winset(C, null, "[padding ? "[w]_padding.parent='';" : ""][w]_child.parent='';[w]_label.parent='';\
[C.dc_MinimapDataString()]")

					//Remove references
					C.dc_Minimaps[C.dc_Minimaps.Find(src)]=null

				else {world.log<<"Tried to Remove non-Initialized Minimap '[id]' from [C]";return}

			else if(istype(A,/list))
				for(var/B in A)
					Remove(B)

			else CRASH("Improper object '[A]' passed to Remove. Expected client, or mob with client.")