dc_MinimapMarker
	var
		id // Unique ID for this Marker. Should not conflict with any Minimap, interface window, or control names

		x // Relative to Minimap's location. Set to an integer for absolute positioning. Set to text for percent-based positioning
		y //  (ex: x=20 would center the marker at 20 pixels to the right of the left side of the Minimap)
		  //  (whereas x="50" would center the marker in the center (50%) of the Minimap)
		width = 1
		height = 1

		color = "#000000" // In the form of #RRGGBB
		image // Set to an image file (eg 'Marker.png')
		imageMode = "" // center, stretch, or tile
		border = "none"

		forceCenter // When using hideEdges on a Minimap, set this to ensure the marker stays with the atom

		//NOTE: Transparent color does not currently display properly
		transparentColor = "" // In the form of #RRGGBB

		//Determined at runtime
		px;py
	New(ID, X,Y, Width, Height, Color,Image,ImageMode,Border,TransparentColor)
		if(ID) id=ID
		if(X) x=X
		if(Y) y=Y
		if(Width) width=Width
		if(Height) height=Height
		if(Color) color=Color
		if(Image) image=Image
		if(ImageMode) imageMode=ImageMode
		if(Border) border=Border
		if(TransparentColor) transparentColor=TransparentColor

		if(!id) CRASH("Marker does not have an ID set")
		if(x==null || y==null) CRASH("Bad Marker coordinates: ([x],[y])")
		if(transparentColor) {world.log<<"Warning: MinimapMarker.transparentColor is currently disabled.";transparentColor=null;}

	proc/Initialize(A,dc_Minimap/P)
		var {client/C;mob/M}
		if(ismob(A)) {M=A;C=M.client}
		else if(istype(A,/client)) {C=A;M=C.mob}

		if(!C) CRASH("Improper object '[A]' passed to Initialize. Expected client, or mob with client.")
		if(!P) CRASH("Need parent Minimap reference for Initialize.")
		if(!C.dc_Minimaps || !(P in C.dc_Minimaps)) CRASH("Cannot Initialize Marker for unInitialized Minimap")

		if(istext(x)) px = P.viewWidth*text2num(x)/100
		else px=x

		if(istext(y)) py = P.viewHeight*text2num(y)/100
		else py=y

		px = round(px - width/2)
		py = round(py - height/2)

		if(image) C<<browse_rsc(image)
		var/w = "[dc_MinimapPrefix][C.dc_Minimaps.Find(P)]"
		C.dc_MinimapData+="[w]_M[id]"
		winset(C, null, C.dc_MinimapDataString())
		winset(C, "[w]_M[id]", "parent=[w]_pane;type=label;pos=[px],[py];size=[width]x[height];anchor1=0,0;background-color=[color];[image ? "image='[image]';" : ""][imageMode ? "image-mode=[imageMode];" : ""][transparentColor ? "is-transparent=true;transparent-color=[transparentColor];" : ""][border ? "border=[border];" : ""]")

	proc/Remove(A,dc_Minimap/P)
		var {client/C;mob/M}
		if(ismob(A)) {M=A;C=M.client}
		else if(istype(A,/client)) {C=A;M=C.mob}

		if(!C) CRASH("Improper object '[A]' passed to Initialize. Expected client, or mob with client.")
		if(!P) CRASH("Need parent Minimap reference for Initialize.")
		if(!C.dc_Minimaps || !(P in C.dc_Minimaps)) CRASH("Cannot Remove Marker for unInitialized Minimap")


		var/w = "[dc_MinimapPrefix][C.dc_Minimaps.Find(P)]"
		C.dc_MinimapData-="[w]_M[id]"
		winset(C, null, "[w]_M[id].parent='';[C.dc_MinimapDataString()]")
