/*
   _____      _             __
  |_   _|    | |           / _|
    | | _ __ | |_ ___ _ __| |_ __ _  ___ ___
    | || '_ \| __/ _ \ '__|  _/ _` |/ __/ _ \
   _| || | | | ||  __/ |  | || (_| | (_|  __/
   \___/_| |_|\__\___|_|  |_| \__,_|\___\___|


  ___  ____       _
  |  \/  (_)     (_)
  | .  . |_ _ __  _ _ __ ___   __ _ _ __
  | |\/| | | '_ \| | '_ ` _ \ / _` | '_ \
  | |  | | | | | | | | | | | | (_| | |_) |
  \_|  |_/_|_| |_|_|_| |_| |_|\__,_| .__/
                                   | |
                                   |_|

    D E M O   8.26.10   BY DARKCAMPAINGER

        VERSION 2.0 - RELEASE 7.30.11

   When running the demo, use the menu to see examples

   Terrain from Karnlor's PGR Public Graphic Resource
   World map image from Blizzard's World of Warcraft

   This demo will walk you through the basics of
    the Interface Minimap library:

       - Preparing your interface for Minimaps
       - Defining Minimaps and Markers
       - Setting up the Minimap Generator
       - Creating Minimaps
       - Registering players to Minimaps
       - Saving generated Minimaps

    Want to know more? Poke through the code, especially
    the Minimap's variables. There's a fair number of
    comments, particularly about usage.
*/

// -------------------------------
// STEP 1: PREPARING YOUR INTERFACE
// --------------------------------------------

//   This library was designed to work without you having to
//   futz with your interface file. All the controls it needs
//   are created at runtime. However, to properly position
//   everything, it needs to be able to set itself up
//            BEFORE ANY WINDOWS ARE MAXIMIZED!
//   Do this by running dc_MinimapPrepare() on all clients
//   in their client/New() proc, as seen below:

client
	New()
		dc_MinimapPrepare()
		.=..()

//    If you maximize windows in client/New(), just make
//    sure dc_MinimapPrepare() is run first. It will
//    automatically restore any maximized windows it finds.




// -------------------------------
// STEP 2: DEFINING MINIMAPS AND MARKERS
// --------------------------------------------

//    The easiest way to use this library is to define all of
//    your Minimaps at compile-time. Minimaps have a lot of
//    goodies built into them to allow you to easily customize
//    their appearance. Here's a basic example:

dc_Minimap/IslandSOS

	// Each Minimap needs to have a unique ID. The ID is used
	// to add, update, and remove Minimaps from players' screens
	id = "MM_IslandSOS"

	// parent holds the name of the interface window it shows in
	parent = "default"

	// These coordinates define the turf area the Minimap
	// encompasses. It's best to keep this area as small as
	// possible to reduce image generation time
	x1 = 1;   y1 = 1
	x2 = 70;  y2 = 44
	z  = 1

	// Use the view variables to define the Minimaps location
	// on the screen. The viewX and viewY variables can be
	// set to simply numbers for absolute positioning, or set
	// to text for relative positioning (following the same
	// syntax as screen_loc). viewAnchor sets how the Minimap
	// moves when the window is resized, (here it is set to
	// follow the bottom-left corner)
	viewX = "WEST"; viewY = "SOUTH-100"
	viewWidth = 100
	viewHeight = 100
	viewAnchor = "0,100"

	// These determine how many pixels each tile takes up in
	// the generated Minimap image. In this case, each tile
	// will be 3x3 pixels.
	scaleX = 3
	scaleY = 3

	// Markers are labels placed over the minimap, often to
	// mark the center point.
	marker = /dc_MinimapMarker/MarkerCenter


dc_MinimapMarker/MarkerCenter
	// Each marker needs a unique ID name
	id = "MarkerCenter"

	// Markers can take plain numbers for absolute positioning
	// on minimap, or a text string for percentage-based
	// positioning.
	x = "50"
	y = "50"

	// Each marker can be any size or color.
	// They are automatically centered at their x,y coords
	width = 4
	height = 4
	color = "#FF0000"


// Here is an example where a pregenerated image is used:

dc_Minimap/WorldMap
	id = "MM_WorldMap"
	parent = "default"

	x1 = 1;    y1 = 1
	x2 = 70;  y2 = 70
	z  = 2

	viewX = "EAST-160"; viewY = "NORTH+10"
	viewWidth = 150
	viewHeight = 150
	viewAnchor = "100,0"

	// By setting the image manually, one will not be generated
	// The image can be much more advanced this way, and avoid
	// the "multiples" limit the generator has
	image = 'WorldMap.jpg'

	// These dimensions set the size of the Minimap image (in pixels).
	// The larger they are, the "closer" the minimap appears.
	// You can define imageWidth/Height *OR* scaleX/Y. As we are
	// using a pregenerated image, we set the image dimensions to be
	// exactly the same as the image
	imageWidth = 326
	imageHeight = 326
	// Note that if you are using the generator and use these instead
	// of scaleX/Y, the image dimensions must be multiples of the turf
	// area dimensions!

	// You can also set the background behind the minimap for
	// when the players are at its edges. If an image, you can set
	// it's display mode to center, stretch, or tile
	backgroundColor = "#000000"
	backgroundImage = 'MapBackground.png'
	backgroundMode = "tile"

	// Padding adds a border around the minimap. It can be set
	// to a solid color or an image. If an image, you can set
	// it's display mode to center, stretch, or tile
	padding=10
	paddingImage = 'MapBorder.jpg'
	paddingMode = "center"

	// You can set how often clients' Minimaps are updated (ticks)
	delay = 4

	marker = /dc_MinimapMarker/MarkerCenterGold

dc_MinimapMarker/MarkerCenterGold
	id = "MarkerCenterGold"
	x = "50"
	y = "50"
	width = 6
	height = 6

	// Markers can also use images (no transparency, however)
	image = 'Marker.png'
	imageMode = "center"

// Here's an example that has only be partially defined.
// The rest will be set when it is created

dc_Minimap/Race
	parent = "default"
	x1 = 1;    y1 = 1
	x2 = 70;  y2 = 70
	z  = 3

	backgroundColor = "#2c82a7"
	padding=4
	paddingColor="#444444"

	// You can set how often clients' Minimaps are updated (ticks)
	delay = 2

	// Marker can also be set to a list of paths
	marker = list(/dc_MinimapMarker/MarkerNorth,/dc_MinimapMarker/MarkerEast,/dc_MinimapMarker/MarkerSouth,/dc_MinimapMarker/MarkerWest)

dc_MinimapMarker/MarkerNorth
	id = "MarkerNorth"
	x = "50"
	y = "40"
	width = 2
	height = 5
	color = "#FF0000"
dc_MinimapMarker/MarkerEast
	id = "MarkerEast"
	x = "60"
	y = "50"
	width = 5
	height = 2
	color = "#FF0000"
dc_MinimapMarker/MarkerSouth
	id = "MarkerSouth"
	x = "50"
	y = "60"
	width = 2
	height = 5
	color = "#FF0000"
dc_MinimapMarker/MarkerWest
	id = "MarkerWest"
	x = "40"
	y = "50"
	width = 5
	height = 2
	color = "#FF0000"


// -------------------------------
// STEP 3: SETTING UP THE GENERATOR
// --------------------------------------------

// The Minimap image generator uses two turf variables to draw
// dc_MinimapState sets the color or icon_state to use when
// drawing the turf, and dc_MinimapPriority sets what "layer"
// it's drawn on (positive only)

turf
	dc_MinimapState="#4a6b30"
	water
		dc_MinimapState="#2c82a7"
	sand
		dc_MinimapState="#efc171"
	rock
		dc_MinimapState="Rock"
		dc_MinimapPriority = 1
	flag
		dc_MinimapState="Flag"
		dc_MinimapPriority = 2

world
	New()
		// If you use icon_states for dc_MinimapState, you have
		// to set the icon the generator grabs them from.
		// It should be set in world/New(), before any Minimaps
		// are created
		dc_MinimapStates = 'dc_MinimapStates.dmi'

// -------------------------------
// STEP 4: CREATING MINIMAPS
// --------------------------------------------

//world
//	New()
		// Now that the Minimaps are all defined, we just have
		// to create them before we use them. world/New() is
		// a good place to do this
		new/dc_Minimap/IslandSOS()
		new/dc_Minimap/WorldMap()

		// We didn't fully define this Minimap, since we're going
		// to use it twice. All of a Minimap's parameters can be
		// set through new() with named arguments (same for Markers).
		// The named arguments are the same as the variable name, but
		// with the first letter capitalized (except for id, which is ID)
		new/dc_Minimap/Race(ID="MM_Race1",ViewX="WEST",ViewY="NORTH",ViewHeight=100,ViewWidth=100,ImageWidth=210,ImageHeight=210,ViewAnchor="0,0")
		new/dc_Minimap/Race(ID="MM_Race2",ViewX="WEST+104",ViewY="NORTH",ViewHeight=50,ViewWidth=50,ImageWidth=280,ImageHeight=280,ViewAnchor="0,0")

		//Call the default proc, or any other world/New() definitions
		..()



// -------------------------------
// STEP 5: REGISTERING PLAYERS TO MINIMAPS
// --------------------------------------------

// Now that all the hard work is done, all we have to do is
// call a process to make our Minimap run!

mob/Player
	icon = 'dc_Demo.dmi'
	icon_state = "Mob"

	Login()
		gotoIsland()

	verb
		gotoIsland()
			// Before showing their new minimap, we call
			// dc_MinimapUnregister() to remove their
			// old one(s). Pass a Minimap ID as the second
			// parameter to remove a specific Minimap
			dc_MinimapUnregister(src)

			loc = locate(10,8,1)
			// Now all we have to do is call dc_MinimapRegister()
			// and pass the mob and Minimap ID! The library will
			// take care of the rest.
			dc_MinimapRegister(src,"MM_IslandSOS")
		gotoWorldMap()
			dc_MinimapUnregister(src)

			loc = locate(35,35,2)
			dc_MinimapRegister(src,"MM_WorldMap")
		gotoRace()
			dc_MinimapUnregister(src)

			loc = locate(38,10,3)
			dc_MinimapRegister(src,"MM_Race1")

			var/mob/Runner/Star/J = locate() in world
			if(J)
				// You can pass a third parameter to specify
				// an atom for the Minimap to follow, instead of
				// the client's mob
				dc_MinimapRegister(src,"MM_Race2",J)


// Want to handle updating the Minimap manually, instead of on a timer?
// Just call dc_MinimapUpdate()! The first parameter must be a client or
// mob with a client. Like with Remove(), pass a Minimap ID
// as the second parameter to update a specific Minimap. The 3rd and 4th
// parameters specify an (x,y) coordinate to center the Minimap on.
//
// WARNING: IF YOU DO NOT PASS AN ID, IT WILL UPDATE *ALL* MINIMAPS
//            TO THE MOB OR COORDINATES **YOU PASSED**!
//
// Here's an example that update the Minimaps whenever the player moves:
// (good for single-player games)

/*

mob/Player/Move()
	.=..()
	dc_MinimapUpdate(src,"MM_IslandSOS")

*/



// -------------------------------
// STEP 6: SAVING GENERATED MINIMAPS
// --------------------------------------------

// You can save processing power and player frustration by compiling all
// of your Minimaps into your resource file. To dump all of the generated
// images to be used as pregenerated images, use a snippet like this:

client/verb/DumpMinimaps()
	if(world.port) return //Only allow the minimaps to be dumped when run offline

	for(var/T in dc_MinimapList)
		var/dc_Minimap/D = dc_MinimapList[T]
		if(D && D.imageGenerated && D.image)
			// Copy the image into the /Minimaps folder
			fcopy(D.image, "Minimaps/[ckeyEx(T)].png")




// -------------------------------
//
// And that's it! The library will handle the rest.
// If you're curious, poke around the code, a fair amount
// of it is commented
//
// If you have any problems, bug reports, suggestions, or
// just general questions, post on my member's page.
// If I'm not lurking, I'm dead! :)
//
//    http://www.byond.com/members/DarkCampainger
//
// --------------------------------------------







//Extra demo stuff. Not related to the library

world
	view = "15x15"
	mob = /mob/Player

turf
	icon = 'dc_Demo.dmi'
	icon_state = "Grass"
	var/randomStates=4
	New()
		if(randomStates)
			var/n=rand(1,randomStates)
			if(n>1) icon_state="[icon_state][n]"
	water
		icon_state = "Water1"
		New() icon_state="Water[((x-1)%3+1)+3*((y-1)%3)]"
	sand
		icon_state = "Sand"
	rock
		icon_state = "Rock"
	flag
		icon_state = "Flag"
		randomStates=0
	setter
		icon_state="Sand"
		dc_MinimapState="#efc171"
		Entered(mob/Runner/M) {if(istype(M)){M.dir=dir}}

mob/Runner
	icon='dc_Demo.dmi'
	icon_state="Runner"
	density=0
	var/delay=6
	New() spawn() while(src) {step(src,dir);sleep(delay);}

	Star/delay=4