/***************
*              *
*  Fog of War  *
*  by Garthor  *
*  2/22/08     *
*              *
***************/

/*
maximum number of foggers that can be registered with fog of war.
the number of areas created is 2**foggers, so use small numbers.
the upper bound is 16, as that would hit up against the area limit
and cause an error, as well as being the maximum for bitwise operators.
*/
var/maxfog = 8

var/list/fogareas = list()
var/list/foggers = list()

proc
	//add a client for fog of war
	fog_add(var/client/C)
		//adding somebody twice will cause problsm
		if(C in foggers)
			return
		//do not allow more than the maximum number of foggers
		if(foggers.len >= maxfog)
			return

		foggers += C

		//List is used to avoid an infinite loop
		var/list/fog = list()
		for(var/area/foggy/F)
			fog += F
		for(var/area/foggy/F in fog)
			F.split(C)

	//remove a client for fog of war
	fog_remove(var/client/C)
		if(C in foggers)
			for(var/area/foggy/F)
				F.merge(C)
			foggers -= C
			for(var/area/foggy/F)
				F.settag()

client
	proc
		//remove the fog for all the turfs in a list
		removefog(var/list/turfs)
			if(!(src in foggers)) return
			if(isturf(turfs))
				turfs = list(turfs)
			for(var/turf/T in turfs)
				var/area/foggy/F = T.loc
				if(!istype(F)) continue
				//if we can't see this turf
				if(src in F.seers)
					var/area/foggy/counterpart = F.getcounterpart(src)
					counterpart.contents += T

		//add fog for all the turfs in a list
		addfog(var/list/turfs)
			if(!(src in foggers)) return
			if(isturf(turfs))
				turfs = list(turfs)
			for(var/turf/T in turfs)
				var/area/foggy/F = T.loc
				if(!istype(F)) continue
				//if we can see this turf
				if(!(src in F.seers))
					var/area/foggy/counterpart = F.getcounterpart(src)
					counterpart.contents += T

		//checks if a client can see something
		cansee(var/area/foggy/F)
			while(F && !isarea(F))
				F = F.loc
			if(!F) return 0
			else
				if(istype(F))
					return !(src in F.seers)
				else
					return 1

area
	foggy
		layer = MOB_LAYER+1
		var/image/fog

		//list of clients who can see this fog (can't see through it)
		var/list/seers = list()
		New(loc,var/list/seers)
			.=..()
			if(seers)
				fog = image('blackHex.dmi',src)
				seers << fog
				src.seers = seers
				//generate the tag
				settag()

		Del()
			//remove the fog, must be done explicitly
			del(fog)
			..()
		proc
			//generate a numerical tag value from a list of seers
			gettag(var/list/seers)
				. = 0
				for(var/v in seers)
					var/i = foggers.Find(v)
					. |= (1 << i-1)

			//sets the tag to the proper value for this area
			settag()
				tag = "fog_[gettag(seers)]"

			/*
			called to get the counterpart area along a client
			(the area that is the same except the client can/can't
			see the other one)
			*/
			getcounterpart(var/client/C)
				var/list/counterpartseers = list()
				if(C in seers)
					counterpartseers = seers - C
				else
					counterpartseers = seers + C
				var/counterparttagnum = gettag(counterpartseers)
				var/area/foggy/counterpart = locate("fog_[counterparttagnum]")

				//if the counterpart can't be found, then there has been a major error
				if(!counterpart)
					world << "fog area [tag] has no counterpart for [C] (can't locate fog_[counterparttagnum])"
					CRASH("fog area [tag] has no counterpart for [C] (can't locate fog_[counterparttagnum])")

				return counterpart

			/*
			called when a new client is added, split this area into
			two new areas: one C can see, and one C can't
			*/
			split(var/client/C)
				var/area/foggy/cansee = new(null,seers+C)
				new /area/foggy(null,seers) //area C can't see
				//transfer all of our turfs to the new cansee area
				for(var/turf/T in src)
					cansee.contents += T
				//remove ourself as we're no longer needed
				del(src)

			/*
			called when a client is removed, merge this area with
			its counterpart that C can/can't see
			*/
			merge(var/client/C)
				world << "merge [tag] [C]"

				//get the counterpart
				var/area/foggy/counterpart = getcounterpart(C)
				world << "counterpart [counterpart.tag]"

				if(C in seers)
					seers -= C

				//merge the two areas
				for(var/turf/T in counterpart)
					contents += T
				del(counterpart)