/*
	-----------------------------------------------------------------

	Deque (/Deque)

	-----------------------------------------------------------------

	A deque is a handy structure with the philosophy that stacks
	and queues are essentially the same type of structure, except
	for the fact that the programmer is limited	to accessing either
	the front or the back of the line.  Deques are defined similarily
	to queues, except with the freedom to push and pop both sides of
	the line.

	-----------------------------------------------------------------

	/Deque
		New(contents, maxlen)
			Initializes the deque with an optional list containing
			the initial contents, and the optional setting of a
			maximum size.  The constructor will make a copy of
			contents.  If 'contents' is neither a list nor null,
			the deque initialize a list containing the single
			item.


		/var/list/contents = list()
			A list of the contents of the stack.  Do not access nor
			modify

		/var/maxlen = -1
			Denotes the maximum amount of items the stack can hold.
			-1 denotes an infinite size.


		/proc/PeekFront()
			Returns the object on the front of the deque

		/proc/PeekBack()
			Returns the object on the back of the deque

		/proc/PushFront(item)
			Adds 'item' into the deque from the front

		/proc/PushBack(item)
			Adds 'item' into the deque from the back

		/proc/PopFront()
			Removes and returns the item at the front.  Returns
			null if the deque is empty.

		/proc/PopBack()
			Removes and returns the item at the back.  Returns
			null if the deque is empty.


		/proc/getSize()
			Returns the size of the queue.

		/proc/isFull()
			Returns 1 if the deque is full; otherwise 0

		/proc/isEmpty()
			Returns 1 if the deque is empty; otherwise 0

		/proc/toString()
			Returns the contents of the deque as a text string
			surrounded by braces

	-----------------------------------------------------------------
*/

Deque
	parent_type = /upListContainer

	proc/PeekFront()
		return src._getStart()

	proc/PeekBack()
		return src._getEnd()

	proc/PopFront()
		return src._popStart()

	proc/PopBack()
		return src._popEnd()

	proc/PushFront(item)
		return src._appendFront(item)

	proc/PushBack(item)
		return src._appendEnd(item)

