
proc
	LinkedList(list/contents)
		return new /LinkedList (contents)

	isValidNode(LinkedList/list, lNode/node)
		return (node.owner == list)

	lNode(LinkedList/newList, data)
		var/lNode/L = new /lNode (newList, data)
		return L

// linked list structure
LinkedList
	var
		lNode/front = null

	New(list/initContents)
		..()

		if(initContents && istype(initContents))
			// insert items backwards at the front to achieve correct order efficiently
			for(var/index = length(initContents), index > 0, index--)
				var/item = initContents[index]
				var/lNode/node = lNode(src, item)
				src.InsertFront(node)

	proc/isEmpty()
		return !src.getFront()

	proc/getSize()
		var/count = 0
		for(var/lNode/n = src.getFront(), n != null, n = n.getNext())
			count++
		return count

	proc/getFront()
		return src.front

	proc/setFront(lNode/_front)
		src.front = _front

	proc/getBack()
		var/lNode/n = src.getFront()
		while(n != null)
			n = n.getNext()
		return n

	proc/Insert(lNode/node)
		src.InsertFront(node)

	proc/RemoveAfter(lNode/node)
		if(!islNode(node) || !isValidNode(src, node))
			upContainers_CRASH("Invalid node removed from LinkedList")

		var/lNode/deleted = node.getNext()
		if(deleted != null)
			node.setNext(deleted.getNext())
			return deleted

		return null

	proc/RemoveFront()
		var/lNode/deleted = src.getFront()
		if(deleted != null)
			src.setFront(deleted.getNext())
			return deleted

		return null

	proc/DeleteAfter(lNode/node)
		if(node == null)
			return 0
		src.RemoveAfter(node)
		node.Destroy()
		return 1

	proc/DeleteFront()
		if(src.getFront() == null)
			return 0
		var/lNode/node = src.RemoveFront()
		node.Destroy()
		return 1

	proc/InsertFront(lNode/node)
		if(!islNode(node))
			node = lNode(src, node)

		node.setNext(src.getFront())
		src.setFront(node)

	proc/InsertAfter(lNode/node, lNode/newNode)
		if(!islNode(newNode))
			newNode = lNode(src, newNode)

		if(!isValidNode(src, node))
			upContainers_CRASH("Invalid node added to LinkedList")

		newNode.setNext(node.getNext())
		node.setNext(newNode)

	proc/InsertEnd(lNode/node)
		if(!islNode(node))
			node = lNode(src, node)

		if(!isValidNode(src, node))
			upContainers_CRASH("Invalid node added to LinkedList")

		if(src.getFront() == null)
			src.InsertFront(node)
		else
			var/lNode/back = src.getBack()
			src.InsertAfter(back)

	proc/toString()
		var/text = "{ "
		for(var/lNode/n = src.getFront(), n != null, n = n.getNext())
			text += "\improper [n.getData()] "
		text += "}"
		return text

lNode
	var
		LinkedList/owner = null
		lNode/next = null
		data = null

	New(LinkedList/_owner, _data, lNode/_next)
		..()

		src.owner = _owner
		src.next = _next
		src.data = _data

	proc/Copy(LinkedList/newList)
		return lNode(newList, src.data)

	proc/Destroy()
		del(src)

	proc/Clear()
		src.setNext(null)

	proc/getData()
		return data

	proc/getNext()
		return src.next

	proc/assignData(_data)
		src.data = _data

	proc/setNext(lNode/_next)
		src.next = _next

