
proc
	DLinkedList(list/contents)
		return new /DLinkedList (contents)

	isValidDLNode(DLinkedList/list, dlNode/node)
		return (node.owner == list)

	dlNode(DLinkedList/newList, data)
		var/dlNode/L = new /dlNode (newList, data)
		return L

// doubly-linked list structure
DLinkedList
	var
		dlNode/front = null
		dlNode/back = null

	New(list/initContents)
		..()

		if(initContents && istype(initContents))
			for(var/item in initContents)
				var/dlNode/node = dlNode(src, item)
				src.InsertEnd(node)

	proc/isEmpty()
		return !(src.getFront() || src.getBack())

	proc/getSize()
		var/count = 0
		for(var/dlNode/n = src.getFront(), n != null, n = n.getNext())
			count++
		return count

	proc/getFront()
		return src.front

	proc/getBack()
		return src.back

	proc/setFront(dlNode/_front)
		src.front = _front

	proc/setBack(dlNode/_back)
		src.back = _back

	proc/Insert(dlNode/node)
		src.InsertFront(node)

	proc/Remove(dlNode/node)
		if(!isdlNode(node) || !isValidDLNode(src, node))
			upContainers_CRASH("Invalid node removed from linked list")

		if(node.getPrev() != null)
			var/dlNode/prev = node.getPrev()
			prev.setNext(node.getNext())
		else
			src.setFront(node.getNext())

		if(node.getNext() != null)
			var/dlNode/next = node.getNext()
			next.setPrev(node.getPrev())
		else
			src.setBack(node.getPrev())

		return node

	proc/Delete(dlNode/node)
		if(node == null)
			return 0
		src.Remove(node)
		node.Destroy()
		return 1

	proc/InsertFront(dlNode/node)
		if(!isdlNode(node))
			node = dlNode(src, node)

		if(src.getFront() == null)
			src.setFront(node)
			src.setBack(node)
			node.setPrev(null)
			node.setNext(null)
		else
			src.InsertBefore(src.getFront(), node)

	proc/InsertEnd(dlNode/node)
		if(!isdlNode(node))
			node = dlNode(src, node)

		if(src.getBack() == null)
			src.InsertFront(node)
		else
			src.InsertAfter(src.getBack(), node)


	proc/InsertAfter(dlNode/node, dlNode/newNode)
		if(!isdlNode(newNode))
			newNode = dlNode(src, newNode)

		if(!isValidDLNode(src, node))
			upContainers_CRASH("Invalid node inserted into linked list")

		newNode.Clear()
		newNode.setPrev(node)
		newNode.setNext(node.getNext())

		if(node.getNext() == null)
			node.setNext(newNode)
			src.setBack(newNode)
		else
			var/dlNode/after = node.getNext()
			after.setPrev(newNode)

		node.setNext(newNode)

	proc/InsertBefore(dlNode/node, dlNode/newNode)
		if(!isdlNode(newNode))
			newNode = dlNode(src, newNode)

		if(!isValidDLNode(src, node))
			upContainers_CRASH("Invalid node inserted into linked list")

		newNode.Clear()
		newNode.setPrev(node.getPrev())
		newNode.setNext(node)

		if(node.getPrev() == null)
			node.setPrev(newNode)
			src.setFront(newNode)
		else
			var/dlNode/before = node.getPrev()
			before.setNext(newNode)

		node.setPrev(newNode)

	proc/toString()
		var/text = "{ "
		for(var/dlNode/n = getFront(), n != null, n = n.getNext())
			text += "\improper [n.getData()] "
		text += "}"
		return text

dlNode
	var
		DLinkedList/owner = null
		dlNode/next = null
		dlNode/prev = null
		data = null

	New(DLinkedList/_owner, _data, dlNode/_next, dlNode/_prev)
		..()

		src.owner = _owner
		src.data = _data
		src.next = _next
		src.prev = _prev

	proc/Copy(DLinkedList/newList)
		return dlNode(newList, src.data)

	proc/Destroy()
		del(src)

	proc/Clear()
		src.setNext(null)
		src.setPrev(null)

	proc/getData()
		return data

	proc/getNext()
		return src.next

	proc/getPrev()
		return src.prev

	proc/assignData(_data)
		src.data = _data

	proc/setNext(dlNode/_next)
		src.next = _next

	proc/setPrev(dlNode/_prev)
		src.prev = _prev


