/*
	-----------------------------------------------------------------

	Queues (/Queue)

	-----------------------------------------------------------------

	A queue is a structure similar to a stack, except the
	objects in a queue are accessed from the front.  The first
	person who has entered a line would be the first person to
	exit.  Objects are Enqueued and Dequeued (added and removed)
	from the queue.

	-----------------------------------------------------------------

	/Queue
		New(contents, maxlen)
			Initializes the queue with an optional list containing
			the initial contents, and the optional setting of a
			maximum size.  The constructor will make a copy of
			contents.  If 'contents' is neither a list nor null,
			the stack initialize a list containing the single
			item.


		/var/list/contents = list()
			A list of the contents of the queue.  Do not access nor
			modify

		/var/maxlen = -1
			Denotes the maximum amount of items the queue can hold.
			-1 denotes an infinite size.


		/proc/getFront() or /proc/Peek()
			Returns the first element of the queue.

		/proc/getBack()
			Returns the last element of the queue.

		/proc/Enqueue(item)
			Inserts 'item' into the queue.  Returns 1 on a successful
			insert; otherwise 0 if the queue is full

		/proc/Dequeue()
			Returns and removes the element at the front of the
			queue.  If the queue is empty, returns null.


		/proc/getSize()
			Returns the size of the queue.

		/proc/isFull()
			Returns 1 if the queue is full; otherwise 0

		/proc/isEmpty()
			Returns 1 if the queue is empty; otherwise 0

		/proc/toString()
			Returns the contents of the queue as a text string
			surrounded by braces

	-----------------------------------------------------------------
*/


Queue
	parent_type = /upListContainer

	proc/Peek()
		return src._getStart()

	proc/getFront()
		return src._getStart()

	proc/getBack()
		return src._getEnd()

	proc/Enqueue(item)
		return src._appendEnd(item)

	proc/Dequeue()
		return src._popStart()

