/*
	-----------------------------------------------------------------

	Stacks (/Stack)

	-----------------------------------------------------------------

	Stacks are used to store a list of objects considering that
	the only method of retrieval of the data is from the top.  A
	stack can be thought as a stack of dishes piled on top of
	each other.  When you finish washing a dish, you put it on
	top of the other dishes.  When removing the dishes, the dish
	that was first washed is the last to be removed. (First in,
	last out)

	-----------------------------------------------------------------

	Ex.

	// create a new stack with the contents { 1 2 3 } and a maximum
	// size of 4
	var/Stack/S = new /Stack (list(1,2,3), 4)

	S.Push(4);	// successfully adds '4' to the stack
	S.Push(5);  // will fail, since the maximum length is 4

	world << S.Pop() // will display and remove '4'

	-----------------------------------------------------------------

	/Stack
		New(contents, maxlen)
			Initializes the stack with an optional list containing
			the initial contents, and the optional setting of a
			maximum size.  The constructor will make a copy of
			contents.  If 'contents' is neither a list nor null,
			the stack initialize a list containing the single
			item.


		/var/list/contents = list()
			A list of the contents of the stack.  Do not access nor
			modify

		/var/maxlen = -1
			Denotes the maximum amount of items the stack can hold.
			-1 denotes an infinite size.


		/proc/getTop() or /proc/Peek()
			Returns the object on the top of the stack

		/proc/Push(item)
			Adds 'item' into the stack

		/proc/Pop()
			Removes and returns the item at the top.  Returns
			null if the stack is empty.


		/proc/getSize()
			Returns the amount of items in the stack

		/proc/isFull()
			If a maximum length was defined (maxlen), returns
			true if the stack is full; otherwise false

		/proc/isEmpty()
			Returns true if the stack is empty; otherwise false

		/proc/toString()
			Returns the contents of the stack as a text string
			surrounded by braces

	-----------------------------------------------------------------
*/

Stack
	parent_type = /upListContainer

	proc/Peek()
		return src._getEnd()

	proc/getTop()
		return src._getEnd()

	proc/Push(item)
		return src._appendEnd()

	proc/Pop()
		return src._popEnd()