
// Abstract container list datum
// Internal.  Do not touch.

upListContainer
	var
		list/contents
		maxlen = -1

	New(list/contents, max=-1)
		..()

		src.maxlen = max
		if(contents != null)
			src.contents = istype(contents) ? contents.Copy() : list(contents)

	proc/getSize()
		if(!src.contents)
			return 0
		return src.contents.len

	proc/isFull()
		if(maxlen == -1)
			return FALSE
		return src.getSize() >= src.maxlen

	proc/isEmpty()
		return src.getSize() == 0

	proc/Copy()
		return src

	proc/toString()
		var/text = "{ "
		for(var/item in src.contents)
			text += "\improper[item] "
		text += "}"
		return text

	// Containers may internally access in a different orientation
	// Do not access directly.  Use functions defined in regular datums
	// 'Start' and 'End' refer to placement in internal list, not in
	// overridden container object.
	proc/_getStart()
		if(src.isEmpty())
			return null
		return src.contents[1]

	proc/_getEnd()
		if(src.isEmpty())
			return null
		return src.contents[src.contents.len]

	proc/_appendEnd(item)
		if(src.isFull())
			return FALSE
		src._contentInitCheck()
		src.contents += item
		return TRUE

	proc/_appendFront(item)
		if(src.isFull())
			return FALSE
		src._contentInitCheck()
		src.contents += item
		for(var/i = contents.len, i > 1, i--)
			src.contents.Swap(i, i-1)
		return TRUE

	proc/_popStart()
		var/item = src._getStart()
		if(item != null)
			src.contents -= item
			src._contentDelCheck()
		return item

	proc/_popEnd()
		var/item = src._getEnd()
		if(item != null)
			src.contents -= item
			src._contentDelCheck()
		return item

	// initialize list when adding to empty list
	proc/_contentInitCheck()
		if(!src.contents)
			src.contents = new /list

	// delete list if list is empty
	proc/_contentDelCheck()
		if(src.contents && !src.contents.len)
			del(src.contents)

	proc/_deleteContents()
		contents = list()


